@@ -1,9 +1,9 @@
-from .helper import PillowTestCase, cached_property
-
 from PIL import Image
 
+from .helper import CachedProperty, assert_image_equal
 
-class TestImagingPaste(PillowTestCase):
+
+class TestImagingPaste:
     masks = {}
     size = 128
 
@@ -23,7 +23,7 @@ class TestImagingPaste(PillowTestCase):
             px[self.size // 2, self.size - 1],
             px[self.size - 1, self.size - 1],
         ]
-        self.assertEqual(actual, expected)
+        assert actual == expected
 
     def assert_9points_paste(self, im, im2, mask, expected):
         im3 = im.copy()
@@ -34,7 +34,7 @@ class TestImagingPaste(PillowTestCase):
         im.paste(im2, mask)
         self.assert_9points_image(im, expected)
 
-    @cached_property
+    @CachedProperty
     def mask_1(self):
         mask = Image.new("1", (self.size, self.size))
         px = mask.load()
@@ -43,11 +43,11 @@ class TestImagingPaste(PillowTestCase):
                 px[y, x] = (x + y) % 2
         return mask
 
-    @cached_property
+    @CachedProperty
     def mask_L(self):
-        return self.gradient_L.transpose(Image.ROTATE_270)
+        return self.gradient_L.transpose(Image.Transpose.ROTATE_270)
 
-    @cached_property
+    @CachedProperty
     def gradient_L(self):
         gradient = Image.new("L", (self.size, self.size))
         px = gradient.load()
@@ -56,38 +56,48 @@ class TestImagingPaste(PillowTestCase):
                 px[y, x] = (x + y) % 255
         return gradient
 
-    @cached_property
+    @CachedProperty
     def gradient_RGB(self):
         return Image.merge(
             "RGB",
             [
                 self.gradient_L,
-                self.gradient_L.transpose(Image.ROTATE_90),
-                self.gradient_L.transpose(Image.ROTATE_180),
+                self.gradient_L.transpose(Image.Transpose.ROTATE_90),
+                self.gradient_L.transpose(Image.Transpose.ROTATE_180),
             ],
         )
 
-    @cached_property
+    @CachedProperty
+    def gradient_LA(self):
+        return Image.merge(
+            "LA",
+            [
+                self.gradient_L,
+                self.gradient_L.transpose(Image.Transpose.ROTATE_90),
+            ],
+        )
+
+    @CachedProperty
     def gradient_RGBA(self):
         return Image.merge(
             "RGBA",
             [
                 self.gradient_L,
-                self.gradient_L.transpose(Image.ROTATE_90),
-                self.gradient_L.transpose(Image.ROTATE_180),
-                self.gradient_L.transpose(Image.ROTATE_270),
+                self.gradient_L.transpose(Image.Transpose.ROTATE_90),
+                self.gradient_L.transpose(Image.Transpose.ROTATE_180),
+                self.gradient_L.transpose(Image.Transpose.ROTATE_270),
             ],
         )
 
-    @cached_property
+    @CachedProperty
     def gradient_RGBa(self):
         return Image.merge(
             "RGBa",
             [
                 self.gradient_L,
-                self.gradient_L.transpose(Image.ROTATE_90),
-                self.gradient_L.transpose(Image.ROTATE_180),
-                self.gradient_L.transpose(Image.ROTATE_270),
+                self.gradient_L.transpose(Image.Transpose.ROTATE_90),
+                self.gradient_L.transpose(Image.Transpose.ROTATE_180),
+                self.gradient_L.transpose(Image.Transpose.ROTATE_270),
             ],
         )
 
@@ -99,7 +109,7 @@ class TestImagingPaste(PillowTestCase):
             im.paste(im2, (12, 23))
 
             im = im.crop((12, 23, im2.width + 12, im2.height + 23))
-            self.assert_image_equal(im, im2)
+            assert_image_equal(im, im2)
 
     def test_image_mask_1(self):
         for mode in ("RGBA", "RGB", "L"):
@@ -145,6 +155,28 @@ class TestImagingPaste(PillowTestCase):
                 ],
             )
 
+    def test_image_mask_LA(self):
+        for mode in ("RGBA", "RGB", "L"):
+            im = Image.new(mode, (200, 200), "white")
+            im2 = getattr(self, "gradient_" + mode)
+
+            self.assert_9points_paste(
+                im,
+                im2,
+                self.gradient_LA,
+                [
+                    (128, 191, 255, 191),
+                    (112, 207, 206, 111),
+                    (128, 254, 128, 1),
+                    (208, 208, 239, 239),
+                    (192, 191, 191, 191),
+                    (207, 207, 112, 113),
+                    (255, 255, 255, 255),
+                    (239, 207, 207, 239),
+                    (255, 191, 128, 191),
+                ],
+            )
+
     def test_image_mask_RGBA(self):
         for mode in ("RGBA", "RGB", "L"):
             im = Image.new(mode, (200, 200), "white")
@@ -199,8 +231,8 @@ class TestImagingPaste(PillowTestCase):
             hist = im.crop(rect).histogram()
             while hist:
                 head, hist = hist[:256], hist[256:]
-                self.assertEqual(head[255], 128 * 128)
-                self.assertEqual(sum(head[:255]), 0)
+                assert head[255] == 128 * 128
+                assert sum(head[:255]) == 0
 
     def test_color_mask_1(self):
         for mode in ("RGBA", "RGB", "L"):
