@@ -1,6 +1,10 @@
+import warnings
+
 import pytest
 
-from PIL import ImageQt
+with warnings.catch_warnings():
+    warnings.simplefilter("ignore", category=DeprecationWarning)
+    from PIL import ImageQt
 
 from .helper import assert_image_equal, assert_image_equal_tofile, hopper
 
